// kinematics table
//     with energy loss in the target
//     ver.5.0

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "defines.h"
#include "commonfunction.h"

void do_calc(char *buf);
double get_pathlength(double theta, int i, int posi, int target, int sw);
double calc_energy(double *mass, double Q, double E, double Ex, double theta, int sw);
double get_theta(int i, int atarget, int *degmax);
int over_threshold(int A, int Z, double E);
double MeV2Ch(double E);
void make_tex_output(void);

FILE *texout;

int main(int argc, char *argv[])
{
  FILE *fp;
  char buf[BUFSIZE];

  // parameter file open 
  fp = fopen(argv[1], "r");
  if(fp == NULL) {
	printf("cannot open file(1)\n");
    return;
  }
  

  // make tex output
  make_tex_output();

  // calc kinematics
  while (1) {
	 if (fgets(buf,sizeof(buf),fp) == NULL)
		  break;

	 do_calc(buf);
  }

  // close tex file 
  fprintf(texout, "\\end{document}\n");
  fclose(fp);
  fclose(texout);
}

void do_calc(char *buf)
{
	char title[BUFSIZE], targettitle[BUFSIZE];
	int readitem, i,j, a[4], z[4], posi, target, degmax;
	double mass[4], massex[4], Q, theta, Energy;
	double Excited,pathlength_before,pathlength_after;
	double Etmp[3], Etotaltrial[MAXTRIAL+1],Etotal,Ecount;

	// read parameters 
	readitem = sscanf(buf, "%s %lf %d",
			title, &Energy, &target);

	 if (readitem != 3) 
		 return;

	 // get atomic number
	 if (!get_atomic_number(title,a,z)) {
		 return;
	 }
	 // get excited energy
	Excited = get_ex_toi(title);

	 // target title
	 if (target == CH2TRGT) {
		 strcpy(targettitle,"CH2");
	 } else if (target == C13TRGT) {
		 strcpy(targettitle,"C13");
	 } else if (target == MLMNTRGT) {
		 strcpy(targettitle,"Melamine");
	 } else if (target == ALMTRGT) {
		 strcpy(targettitle, "Aluminium");
	 }

	 // calc mass 
	 for (i = 0; i < 4; i++) {
		 massex[i] = calc_massex(a[i], z[i]);
		 mass[i] = calc_mass(a[i],z[i],massex[i]);
	 }
	 Q = massex[0]+massex[1]-massex[2]-massex[3];

	 // print template
	 printf("   %s, Q = %3.2lf, E = %3.2lf, Ex = %3.2lf\n", title, Q, Energy, Excited);
	 fprintf(texout, "\\begin{table}[H]\n\\begin{center}\n\\begin{tabular}{llll}\n\\toprule\n");
	 fprintf(texout, "\\multicolumn{4}{c}{%s in %s}\\\\\n", title, targettitle);
	 fprintf(texout, "\\multicolumn{4}{c}{Q-value %3.2lf, E = %3.2lf, Ex = %3.2lf}\\\\\n", 
			 Q, Energy, Excited);
	 fprintf(texout, "deg & total energy & E - dE & dE \\\\\n\\midrule\n");

	 // calculation
	 i = 0;
	 while (1) {
		 theta = get_theta(i,target,&degmax);
		 i++;
		 if ((theta == ERR) || (degmax == 0))
			 break;

		 if (theta > 60) posi = LADDER_POSI2;
		 else posi = LADDER_POSI1;

		 for (j = 0; j <= MAXTRIAL; j++) {
			 // get path length
			 pathlength_before = get_pathlength(theta,j,posi,target,BEFORE);
			 pathlength_after = get_pathlength(theta,j,posi,target,AFTER);
			 if ((pathlength_after == ERR) || (pathlength_before == ERR))
				 continue;

			 // calc energy loss in the target
			 Etmp[0] = bethe(Energy,z[0],mass[0],pathlength_before,TRUE,target); // before
			 Etmp[1] = calc_energy(mass,Q,Etmp[0],Excited,theta,TRUE);  // scattering
			 Etotaltrial[j] = bethe(Etmp[1],z[2],mass[2],pathlength_after,TRUE,target); //after
		 }
		 // get the mean of total energy
		 Etotal = 0;
		 for (j = 0; j <= MAXTRIAL; j++) {
			 Etotal += Etotaltrial[j];
		 }
		 Etotal = Etotal/(MAXTRIAL + 1);

		 // calc energy loss in the delta E counter
		 if (!over_threshold(a[2],z[2],Etotal)) continue;
		 Ecount = bethe(Etotal,z[2],mass[2],dETHICKNESS,TRUE,SILICON);
		 // calc energy loss in the E counter's dead zone
		 Ecount = bethe(Ecount,z[2],mass[2],DEADTHICKNESS,TRUE,SILICON);

// result
		 fprintf(texout, "%3.1lf & %5.4le & %5.4le & %5.4le \\\\\n",
				 theta, Etotal, Ecount, Etotal-Ecount);
	 }
	 fprintf(texout, "\\bottomrule\n\\end{tabular}\n\\end{center}\n\\end{table}\n");
}

double get_pathlength(double theta, int i, int posi, int target, int sw)
{
	double thickness, pathlength;

	// get thickness
	if (target == CH2TRGT) {
		thickness = CH2THICKNESS;
	} else if (target == ALMTRGT) {
		thickness = ALMTHICKNESS;
	} else if (target == MLMNTRGT) {
		thickness = MLMTHICKNESS;
	} else if (target == C13TRGT) {
		thickness = C13THICKNESS;
	} else {
		return ERR;
	}

	// before
	if (sw == BEFORE) {
		if (posi == LADDER_POSI1) {
			pathlength = thickness*((double)i/(double)MAXTRIAL);
			return pathlength;
		} else if (posi == LADDER_POSI2) {
			pathlength = thickness*((double)i/(double)MAXTRIAL)*(1/cos(TRGTANGL*PI/180));
			return pathlength;
		} else if (posi == LADDER_POSI3) {
			pathlength = thickness*((double)i/(double)MAXTRIAL)*(1/cos(TRGTANGL*PI/180));
			return pathlength;
		} else {
			return ERR;
		}
	}

	// after
	if (sw == AFTER) {
		if (posi == LADDER_POSI1) {
			pathlength = thickness*(1-(double)i/(double)MAXTRIAL)*(1/cos(theta*PI/180));
			return pathlength;
		} else if (posi == LADDER_POSI2) {
			pathlength = thickness*(1 - ((double)i/(double)MAXTRIAL)/(sqrt(2)))/cos((theta-TRGTANGL)*PI/180);
			return pathlength;
		} else if (posi == LADDER_POSI3) {
			pathlength = thickness*((double)i/(double)MAXTRIAL)/(fabs(cos((theta+TRGTANGL)*PI/180)));
			return pathlength;
		} else {
			return ERR;
		}
	}
}

double calc_energy(double *mass, double Q, double E, double Ex, double theta, int sw)
{
	double A,B,C,v,Ep,Ey;

	theta = theta*PI/1.8e2;

// calc energy in Newton Dynamics 
	A = mass[2]*(mass[2]+mass[3]);
	B = sqrt(2.0e0*mass[0]*E)*mass[2]*cos(theta);
	C = 2.0e0*mass[0]*E - 2.0e0*mass[3]*(E+Q-Ex);
	 
	if (B*B - A*C < 0) return 0;

	if (sw == TRUE) {
		v = (B+sqrt(B*B-A*C))/A;
	} else if (sw == FALSE) {
		v = (B-sqrt(B*B-A*C))/A;
	}

	Ep = 5.0e-1*mass[2]*v*v;

	// check energy conservation 
	Ey = (1/(2.0e0*mass[3]))*(2.0e0*mass[0]*E+2.0e0*mass[2]*Ep)
		-(sqrt(2.0e0*mass[0]*E*2.0e0*mass[2]*Ep)/(mass[3]))*cos(theta)
		+Ex;

	if (fabs((E+Q)-(Ey+Ep)) < 1e-3) {
		return Ep;
	} else { 
		return 0;
	}
}

// get theta from array
double get_theta(int i, int atarget, int *degmax)
{
	double theta;

	if (atarget == ALMTRGT) {
		*degmax = (sizeof(theta_list_Al)/sizeof(double));
		if (i >= *degmax) return ERR;
		return theta_list_Al[i];
	} else if (atarget == C13TRGT) {
		*degmax = (sizeof(theta_list_C13)/sizeof(double));
		if (i >= *degmax) return ERR;
		return theta_list_C13[i];
	} else if (atarget == CH2TRGT) {
		*degmax = (sizeof(theta_list_CH2)/sizeof(double));
		if (i >= *degmax) return ERR;
		return theta_list_CH2[i];
	} else if (atarget == MLMNTRGT) {
		*degmax = (sizeof(theta_list_Melamine)/sizeof(double));
		if (i >= *degmax) return ERR;
		return theta_list_Melamine[i];
	} else {
		*degmax = THETAMAX/DELTATHETA+1;
		theta = i*DELTATHETA;
		if (theta > THETAMAX) return ERR;
		return theta;
	}
}

// return over the threshold in the delta E counter
int over_threshold(int A, int Z, double E)
{
	double threshold = 0.0e0;
	if ((A == 1) && (Z == 1)) {
		threshold = 1.3e0; // proton
	} else if ((A == 2) && (Z == 1)) {
		threshold = 1.6e0; // deutron
	} else if ((A == 3) && (Z == 1)) {
		threshold = 1.8e0; // triton
	} else if ((A == 3) && (Z == 2)) {
		threshold = 4.6e0; // 3He
	} else if ((A == 4) && (Z == 2)) {
		threshold = 5.2e0; // alpha
	} else {
		return FALSE;
	}

	if (E - threshold > 0)
		return TRUE;
	else 
		return FALSE;
}

double MeV2Ch(double E)
{
	double channel;
	channel = fabs((E-MEVSHIFT)/MEVPERCH);
	return channel;
}

void make_tex_output(void)
{
	texout = fopen("table.tex", "w");
// template
	fprintf(texout, "\\documentclass[twocolumn]{jsarticle}\n");
	fprintf(texout, "\\usepackage{amsmath}\n\\usepackage{booktabs}\n\\usepackage{float}\n");
	fprintf(texout, "\\renewcommand{\\tablename}{Table.}\n");
	fprintf(texout, "\\begin{document}\n");
	fprintf(texout, "\\title{Kinematics Tables}\n\\date{}\n\\maketitle\n");
}
